from typing import TypedDict

from .schemas import AirAttribute, CabinClass


class AirAttributeFlightNumberRequestDataType(TypedDict):
    cabin_class: CabinClass
    carrier: str
    departure: str
    arrival: str
    departure_date: str
    flight_no: str


class SliceDeparture(TypedDict):
    code: str
    date: str


class SliceArrival(TypedDict):
    code: str


class SliceType(TypedDict):
    departure: SliceDeparture
    arrival: SliceArrival


class AirAttributeRouteRequestDataType(TypedDict):
    cabin_class: CabinClass
    slices: SliceType
    passengers: int


class AirAttributeAPI:
    async def get_airattributes_by_flight_number(self, data: dict) -> AirAttribute:
        url = f"{self.BASE_URL}/search/amenities/"
        async with self.session.post(url, json=data) as response:
            res = await self.parse_response(response)
            return AirAttribute.from_json(res)

    async def get_airattributes_by_route(self, data: dict) -> AirAttribute:
        url = f"{self.BASE_URL}/amenities/search/route/"
        async with self.session.post(url, json=data) as response:
            res = await self.parse_response(response)
            return AirAttribute.from_json(res)



from dataclasses import dataclass
from dataclasses import fields as dataclass_fields
from datetime import datetime
from decimal import Decimal
from typing import Any, List, Literal, Optional, Union, get_args, get_origin

PassengerType = Literal["adult", "child", "infant"]
ServiceMetaType = Literal["checked", "carry_on"]
CabinClass = Literal["first", "business", "premium_economy", "economy"]


class FlylineBaseDataClass:
    @classmethod
    def from_json(cls, json_data: Union[dict, list]) -> Any:
        def convert_dict2dataclass(field_type, v):
            if hasattr(field_type, "from_json"):
                return field_type.from_json(v)
            elif field_type is Decimal:
                return Decimal(str(v))
            elif field_type is datetime:
                return datetime.fromisoformat(v)
            else:
                return v

        ret = []
        should_be_flatten = False
        if not isinstance(json_data, list):
            json_data = [json_data]
            should_be_flatten = True

        for ele_data in json_data:
            cdata = {}
            for field in dataclass_fields(cls):
                key = field.name
                value = ele_data.get(key, None)
                if value is not None:
                    if get_origin(field.type) is list:
                        cdata[key] = [convert_dict2dataclass(get_args(field.type)[0], item) for item in value]
                    else:
                        cdata[key] = convert_dict2dataclass(field.type, value)
                else:
                    cdata[key] = None
            ret.append(cls(**cdata))

        return ret[0] if should_be_flatten else ret


@dataclass(frozen=True)
class PaginationMeta(FlylineBaseDataClass):
    limit: int
    after: Optional[str] = None
    before: Optional[str] = None


# Basic & Air Resources Schemas
@dataclass(frozen=True)
class Aircraft(FlylineBaseDataClass):
    iata_code: str
    name: str


@dataclass(frozen=True)
class AircraftList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Aircraft]


@dataclass(frozen=True)
class Airline(FlylineBaseDataClass):
    iata_code: str
    name: str


@dataclass(frozen=True)
class AirlineList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Airline]


@dataclass(frozen=True)
class City(FlylineBaseDataClass):
    iata_code: str
    name: str
    iata_country_code: str


@dataclass(frozen=True)
class CityList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[City]


@dataclass(frozen=True)
class Airport(FlylineBaseDataClass):
    iata_code: str
    name: str
    iata_country_code: str
    latitude: Decimal
    longitude: Decimal
    icao_code: str
    time_zone: str
    city: City


@dataclass(frozen=True)
class AirportList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Airport]


@dataclass(frozen=True)
class SeatType(FlylineBaseDataClass):
    display_text: str
    pitch: str
    width: str


@dataclass(frozen=True)
class SeatTypeList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[SeatType]


@dataclass(frozen=True)
class SeatLayout(FlylineBaseDataClass):
    display_text: str


@dataclass(frozen=True)
class SeatLayoutList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[SeatLayout]


@dataclass(frozen=True)
class Food(FlylineBaseDataClass):
    display_text: str
    cost: str


@dataclass(frozen=True)
class FoodList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Food]


@dataclass(frozen=True)
class Beverage(FlylineBaseDataClass):
    display_text: str
    type: str
    alcoholic_cost: str
    nonalcoholic_cost: str


@dataclass(frozen=True)
class BeverageList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Beverage]


@dataclass(frozen=True)
class Entertainment(FlylineBaseDataClass):
    display_text: str


@dataclass(frozen=True)
class EntertainmentList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Entertainment]


@dataclass(frozen=True)
class Wifi(FlylineBaseDataClass):
    display_text: str
    quality: str
    cost: str


@dataclass(frozen=True)
class WifiList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Wifi]


@dataclass(frozen=True)
class Power(FlylineBaseDataClass):
    display_text: str
    multiple_at_seat: str
    usb_port: str
    power_outlet: str


@dataclass(frozen=True)
class PowerList(FlylineBaseDataClass):
    pagination: PaginationMeta
    data: List[Power]


# Air Fare Schemas
@dataclass(frozen=True)
class Passenger(FlylineBaseDataClass):
    id: str
    age: int
    type: PassengerType


@dataclass(frozen=True)
class SegmentBaggage(FlylineBaseDataClass):
    type: ServiceMetaType
    quantity: int


@dataclass(frozen=True)
class SegmentPassenger(FlylineBaseDataClass):
    passenger_id: str
    baggages: List[SegmentBaggage]


@dataclass(frozen=True)
class Segment(FlylineBaseDataClass):
    id: str
    aircraft: str
    origin: str
    origin_terminal: str
    departing_at: datetime
    destination: str
    destination_terminal: str
    arriving_at: datetime
    distance: int
    duration: str
    marketing_carrier: str
    marketing_carrier_flight_number: str
    operating_carrier: str
    operating_carrier_flight_number: str
    passengers: List[SegmentPassenger]


@dataclass(frozen=True)
class Slice(FlylineBaseDataClass):
    origin: str
    destination: str
    duration: str
    segments: List[Segment]


@dataclass(frozen=True)
class FareRulesTicketing(FlylineBaseDataClass):
    endorsements: List[str]


@dataclass(frozen=True)
class FareRules(FlylineBaseDataClass):
    refundable: bool
    corporate: bool
    change_fee: bool
    cancel_fee: bool
    cancellation_change_fees: str
    seat_selection: str
    boarding_zone: str
    points_mileage: str
    ticketing: FareRulesTicketing


@dataclass(frozen=True)
class FareSegmentCabinAmenities(FlylineBaseDataClass):
    food: Food
    beverage: Beverage
    entertainment: Entertainment
    wifi: Wifi
    power: Power


@dataclass(frozen=True)
class FareSegmentCabinAttributes(FlylineBaseDataClass):
    seat_attributes: SeatType
    seat_layout: SeatLayout


@dataclass(frozen=True)
class FareSegment(FlylineBaseDataClass):
    segment_id: str
    cabin_amenities: FareSegmentCabinAmenities
    cabin_attributes: FareSegmentCabinAttributes


@dataclass(frozen=True)
class Fare(FlylineBaseDataClass):
    cabin_class: str
    cabin_name: str
    fare_class: str
    fare_name: str
    fare_basis_code: str
    fare_type: str
    fare_rules: FareRules
    segments: List[FareSegment]
    passengers: List[str]


@dataclass(frozen=True)
class Offer(FlylineBaseDataClass):
    id: str
    base_amount: Decimal
    base_currency: str
    tax_amount: Decimal
    tax_currency: str
    total_amount: Decimal
    total_currency: str
    passengers: List[Passenger]
    owner: str
    slices: List[Slice]
    fares: List[Fare]
    expires_at: datetime
    ticket_time_limit: datetime
    max_connections: int


@dataclass(frozen=True)
class AirFare(FlylineBaseDataClass):
    live_mode: bool
    count: int
    airports: List[Airport]
    aircraft: List[Aircraft]
    carriers: List[Airline]
    offers: List[Offer]


# Air Attribute API Schema
@dataclass(frozen=True)
class TripAttributeSegment(FlylineBaseDataClass):
    origin: str
    destination: str
    aircraft: str
    flight_no: str
    cabin_amenities: FareSegmentCabinAmenities
    cabin_attributes: FareSegmentCabinAttributes


@dataclass(frozen=True)
class TripAttributeSlice(FlylineBaseDataClass):
    origin: str
    destination: str
    segments: List[TripAttributeSegment]


@dataclass(frozen=True)
class FareAttribute(FlylineBaseDataClass):
    baggage_rules: str
    cancellation_change_fees: str
    seat_selection: str
    boarding_zone: str
    points_mileage: str


@dataclass(frozen=True)
class TripAttribute(FlylineBaseDataClass):
    carrier: str
    slices: List[TripAttributeSlice]
    fare_attributes: FareAttribute


@dataclass(frozen=True)
class AirAttribute(FlylineBaseDataClass):
    cabin_class: CabinClass
    carriers: List[Airline]
    airports: List[Airport]
    aircraft: List[Aircraft]
    trip_attributes: List[TripAttribute]


# Schedule API Schemas
@dataclass(frozen=True)
class ScheduleDeparture(FlylineBaseDataClass):
    airport_code: str
    city: str
    country_code: str
    time_scheduled: datetime
    time_expected: Optional[datetime] = None
    time_actual: Optional[datetime] = None
    terminal: Optional[str] = None


@dataclass(frozen=True)
class ScheduleArrival(FlylineBaseDataClass):
    airport_code: str
    city: str
    country_code: str
    time_scheduled: datetime
    time_expected: Optional[datetime] = None
    time_actual: Optional[datetime] = None
    terminal: Optional[str] = None


@dataclass(frozen=True)
class Schedule(FlylineBaseDataClass):
    flight_status: str
    flight_no: str
    aircraft: str
    distance: int
    departure: ScheduleDeparture
    arrival: ScheduleArrival


# Seat Map API Schemas
@dataclass(frozen=True)
class Seat(FlylineBaseDataClass):
    designator: str
    description: str
    position_type: str


@dataclass(frozen=True)
class SeatMapByClass(FlylineBaseDataClass):
    seat_count: int
    cabin_class: str
    data: List[Seat]


@dataclass(frozen=True)
class SeatMap(FlylineBaseDataClass):
    display_text: str
    airline_code: str
    airline_name: str
    aircraft_code: str
    # seat_map: str
    # seat_map_key: str
    overview: str
    seats: List[SeatMapByClass]
    traveler_photos: Optional[List[str]] = None


@dataclass(frozen=True)
class CabinClassMapping(FlylineBaseDataClass):
    carrier: str
    cabin_class: str
    cabin_class_code: str
    fare_class_codes: List[str]



class FlylineBadRequest(Exception):
    pass


class AuthenticationFailed(Exception):
    def __init__(self, *args, **kwargs):
        self.message = "Authentication Failed"
        super().__init__(self.message)


class FlylineItemNotFound(Exception):
    def __init__(self, *args, **kwargs):
        self.message = "Requested data not found"
        super().__init__(self.message)


class FlylineServerError(Exception):
    def __init__(self, *args, **kwargs):
        self.message = "FlylineServer Error"
        super().__init__(self.message)


class FlylinePayloadError(Exception):
    def __init__(self, *args, **kwargs):
        self.message = "Payload Error"
        super().__init__(self.message)



from typing import Any, Dict, List, Optional, Tuple, Union

from .errors import FlylinePayloadError
from .schemas import (
    Aircraft,
    AircraftList,
    Airline,
    AirlineList,
    Airport,
    AirportList,
    Beverage,
    BeverageList,
    CabinClassMapping,
    City,
    CityList,
    Entertainment,
    EntertainmentList,
    Food,
    FoodList,
    PaginationMeta,
    Power,
    PowerList,
    SeatLayout,
    SeatLayoutList,
    SeatType,
    SeatTypeList,
    Wifi,
    WifiList,
)

Resource = Union[Aircraft, Airline, Airport, Beverage, City, Entertainment, Food, Power, SeatType, SeatLayout, Wifi]


class AirResourcesAPI:
    async def _get_resources(self, _class: Any, endpoint: str) -> Tuple[PaginationMeta, List[Resource]]:
        async with self.session.get(f"{self.BASE_URL}/{endpoint}/") as response:
            res = await self.parse_response(response)
            try:
                pagination_meta = res["meta"]
                data = res["data"]
            except KeyError:
                raise FlylinePayloadError()
            else:
                return PaginationMeta.from_json(pagination_meta), _class.from_json(data)

    async def _get_resource(self, _class: Any, endpoint: str, code: str) -> Optional[Resource]:
        async with self.session.get(f"{self.BASE_URL}/{endpoint}/{code}/") as response:
            res = await self.parse_response(response)
            try:
                data = res["data"]
            except KeyError:
                raise FlylinePayloadError()
            return _class.from_json(data)

    async def get_aircrafts(self) -> AircraftList:
        res = await self._get_resources(_class=Aircraft, endpoint="aircraft")
        return AircraftList(pagination=res[0], data=res[1])

    async def get_aircraft(self, iata_code: str) -> Optional[Aircraft]:
        return await self._get_resource(_class=Aircraft, endpoint="aircraft", code=iata_code)

    async def get_airlines(self) -> AirlineList:
        res = await self._get_resources(_class=Airline, endpoint="airlines")
        return AirlineList(pagination=res[0], data=res[1])

    async def get_airline(self, iata_code: str) -> Airline:
        return await self._get_resource(_class=Airline, endpoint="airlines", code=iata_code)

    async def get_airports(self) -> AirportList:
        res = await self._get_resources(_class=Airport, endpoint="airports")
        return AirportList(pagination=res[0], data=res[1])

    async def get_airport(self, iata_code: str) -> Optional[Airport]:
        return await self._get_resource(_class=Airport, endpoint="airports", code=iata_code)

    async def get_airports_by_city(self, city_iata_code: str) -> List[Airport]:
        async with self.session.get(f"{self.BASE_URL}/cities/{city_iata_code}/airports/") as response:
            res = await self.parse_response(response)
            return Airport.from_json(res.get("data", []))

    async def get_cities(self) -> CityList:
        res = await self._get_resources(_class=City, endpoint="cities")
        return CityList(pagination=res[0], data=res[1])

    async def get_city(self, iata_code: str) -> Optional[City]:
        return await self._get_resource(_class=City, endpoint="cities", code=iata_code)

    async def get_cabin_class_mapping(
        self, carrier: Optional[str] = None, cabin_class: Optional[str] = None
    ) -> Dict[str, List[CabinClassMapping]]:
        params = {}
        if carrier:
            params["carrier"] = carrier
        if cabin_class:
            params["cabin_class"] = cabin_class

        async with self.session.get(f"{self.BASE_URL}/cabin-booking/", params=params) as response:
            res = await self.parse_response(response)
            return {carrier: CabinClassMapping.from_json(data) for carrier, data in res.items()}

    async def get_seat_types(self) -> SeatTypeList:
        res = await self._get_resources(_class=SeatType, endpoint="seats")
        return SeatTypeList(pagination=res[0], data=res[1])

    async def get_seat_layouts(self) -> SeatLayoutList:
        res = await self._get_resources(_class=SeatLayout, endpoint="layouts")
        return SeatLayoutList(pagination=res[0], data=res[1])

    async def get_foods(self) -> FoodList:
        res = await self._get_resources(_class=Food, endpoint="foods")
        return FoodList(pagination=res[0], data=res[1])

    async def get_beverages(self) -> BeverageList:
        res = await self._get_resources(_class=Beverage, endpoint="beverages")
        return BeverageList(pagination=res[0], data=res[1])

    async def get_entertainments(self) -> EntertainmentList:
        res = await self._get_resources(_class=Entertainment, endpoint="entertainments")
        return EntertainmentList(pagination=res[0], data=res[1])

    async def get_wifis(self) -> WifiList:
        res = await self._get_resources(_class=Wifi, endpoint="wifis")
        return WifiList(pagination=res[0], data=res[1])

    async def get_powers(self) -> PowerList:
        res = await self._get_resources(_class=Power, endpoint="powers")
        return PowerList(pagination=res[0], data=res[1])



from . import (
    AirAttributeAPI,
    AirFareAPI,
    AirResourcesAPI,
    AirScheduleAPI,
    AirSeatMapAPI,
    FlylineBaseClient,
)


class FlylineClient(AirFareAPI, AirSeatMapAPI, AirAttributeAPI, AirScheduleAPI, AirResourcesAPI, FlylineBaseClient):
    pass



import asyncio

from aiohttp import ClientResponse, ClientSession

from .errors import (
    AuthenticationFailed,
    FlylineBadRequest,
    FlylineItemNotFound,
    FlylineServerError,
)


class FlylineBaseClient:
    BASE_URL = "https://api.flyline.io/api"

    def __init__(self, key: str):
        self._key = key
        headers = {"Authorization": f"FToken {self._key}"}
        self._loop = asyncio.get_event_loop()
        self.session = ClientSession(headers=headers, loop=self._loop)

    @classmethod
    async def parse_response(cls, response: ClientResponse):
        if response.status == 401:
            raise AuthenticationFailed()
        elif response.status == 404:
            raise FlylineItemNotFound()
        elif response.status == 500:
            raise FlylineServerError()

        res = await response.json()
        if response.status == 400:
            raise FlylineBadRequest(res)
        else:
            return res

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.session.close()



from typing import List, Optional

from .schemas import Schedule


class AirScheduleAPI:
    async def get_schedules_by_flight_number(self, airline: str, date: str, flight_number: str) -> Optional[Schedule]:
        data = {
            "airline": airline,
            "date": date,
            "flight_number": flight_number,
        }
        async with self.session.post(f"{self.BASE_URL}/schedule-flight/", json=data) as response:
            res = await self.parse_response(response)
            if "message" in res:
                return None
            return Schedule.from_json(res)

    async def get_schedules_by_route(
        self, airline: str, origin: str, destination: str, date: str
    ) -> Optional[List[Schedule]]:
        data = {
            "airline": airline,
            "origin": origin,
            "destination": destination,
            "date": date,
        }
        async with self.session.post(f"{self.BASE_URL}/schedule/", json=data) as response:
            res = await self.parse_response(response)
            if "message" in res:
                return None
            return Schedule.from_json(res)



from typing import List, Literal, TypedDict

CabinClass = Literal["first", "business", "premium_economy", "economy", "basic_ecnomoy"]


class AirFareRequestSlice(TypedDict):
    pass


class AirFareRequestPassenger(TypedDict):
    pass


class AirFareRequestMaxPrice(TypedDict):
    price: str
    currency: str


class AirFareRequestCorporateAccount(TypedDict):
    pass


class AirFareRequest(TypedDict):
    cabin_class: CabinClass
    slices: List[AirFareRequestSlice]
    passengers: List[AirFareRequestPassenger]
    max_price: AirFareRequestMaxPrice
    permitted_carriers: List[str]
    sort_by: str
    corporate_accounts: List[AirFareRequestCorporateAccount]



from typing import List

from .schemas import SeatMap


class AirSeatMapAPI:
    async def get_seat_map(self, carrier: str, aircraft: str) -> List[SeatMap]:
        data = {"carrier": carrier, "aircraft": aircraft}
        async with self.session.post(f"{self.BASE_URL}/seat-maps/", json=data) as response:
            res = await self.parse_response(response)
            return SeatMap.from_json(res)



from .airattributes import AirAttributeAPI  # noqa
from .airfare import AirFareAPI  # noqa
from .airresources import AirResourcesAPI  # noqa
from .airschedules import AirScheduleAPI  # noqa
from .airseatmaps import AirSeatMapAPI  # noqa
from .flylinebaseclient import FlylineBaseClient  # noqa
from .flylineclient import FlylineClient  # noqa



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flyline']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0']

setup_kwargs = {
    'name': 'flyline',
    'version': '0.1.3',
    'description': 'Flyline Client Libraray',
    'long_description': '# flyline_python\nFlyline Python Library\n\n## Install\n```python\npip install flyline\n```\n\n## Getting Started\n\n```python\nimport asyncio\nfrom flyline import FlylineClient\n\nasync def main():\n    key = \'test_***\'\n    async with FlylineClient(key=key) as client:\n        data = {\n            "cabin_class": "economy",\n            "slices": [\n                {\n                    "departure": {"code": "DFW", "date": "2021-06-12"},\n                    "arrival": {"code": "LAX"}\n                }\n            ],\n            "passengers": [{"age": 27}],\n        }\n        seat_map = await client.get_airfares(data=data)\n        print(seat_map)\n\n\nloop = asyncio.get_event_loop()\nloop.run_until_complete(main())\n```\n\n## Endpoints\n### AirFare API\n```python\n    client.get_airfares()\n```\n\n### AirAttribute API\n```python\n    client.get_airattributes_by_flight_number()\n    client.get_airattributes_by_route()\n```\n\n### AirSchedule API\n```python\n    client.get_schedules_by_flight_number()\n    client.get_schedules_by_route()\n\n```\n\n### AirSeatMap API\n```python\n    client.get_seat_map()\n```\n\n### AirResources API\n```python\n    client.get_aircrafts()\n    client.get_aircraft()\n    client.get_airlines()\n    client.get_airline()\n    client.get_airports()\n    client.get_airport()\n    client.get_airports_by_city()\n    client.get_cities()\n    client.get_city()\n    client.get_cabin_class_mapping()\n    client.get_seat_types()\n    client.get_seat_layouts()\n    client.get_foods()\n    client.get_beverages()\n    client.get_entertainments()\n    client.get_wifis()\n    client.get_powers()\n```',
    'author': 'Flyline Development Team',
    'author_email': 'development@flyline.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from .schemas import AirFare

# from .types import AirFareRequest


class AirFareAPI:
    async def get_airfares(self, data: dict) -> AirFare:
        async with self.session.post(f"{self.BASE_URL}/flights/shop/", json=data) as response:
            res = await self.parse_response(response)
            return AirFare.from_json(res)
